# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CounterUpdateParams"]


class CounterUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    name: Required[str]
    """Descriptive name for the Counter."""

    unit: Required[str]
    """
    User defined label for units shown on Bill line items, and indicating to your
    customers what they are being charged for.
    """

    code: str
    """Code for the Counter. A unique short code to identify the Counter."""

    product_id: Annotated[str, PropertyInfo(alias="productId")]
    """UUID of the product the Counter belongs to.

    _(Optional)_ - if left blank, the Counter is Global. A Global Counter can be
    used to price Plans or Plan Templates belonging to any Product.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["TransactionListParams"]


class TransactionListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """`nextToken` for multi page retrievals.

    A token for retrieving the next page of transactions. You'll get this from the
    response to your request.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """The maximum number of transactions to return per page."""

    transaction_type_id: Annotated[Optional[str], PropertyInfo(alias="transactionTypeId")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UsageSubmitParams", "Measurement"]


class UsageSubmitParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    measurements: Required[Iterable[Measurement]]
    """Request containing the usage data measurements for submission."""


class Measurement(TypedDict, total=False):
    account: Required[str]
    """Code of the Account the measurement is for."""

    meter: Required[str]
    """Short code identifying the Meter the measurement is for."""

    ts: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """Timestamp for the measurement _(in ISO 8601 format)_."""

    cost: Dict[str, float]
    """'cost' values"""

    ets: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """End timestamp for the measurement _(in ISO 8601 format)_. _(Optional)_.

    Can be used in the case a usage event needs to have an explicit start and end
    rather than being instantaneous.
    """

    income: Dict[str, float]
    """'income' values"""

    measure: Dict[str, float]
    """'measure' values"""

    metadata: Dict[str, str]
    """'metadata' values"""

    other: Dict[str, str]
    """'other' values"""

    uid: str
    """Unique ID for this measurement."""

    what: Dict[str, str]
    """'what' values"""

    where: Dict[str, str]
    """'where' values"""

    who: Dict[str, str]
    """'who' values"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ScheduledEventConfigurationCreateParams"]


class ScheduledEventConfigurationCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    entity: Required[str]
    """
    The referenced configuration or billing entity for which the desired scheduled
    Event will trigger.
    """

    field: Required[str]
    """
    A DateTime field for which the desired scheduled Event will trigger - this must
    be a DateTime field on the referenced billing or configuration entity.
    """

    name: Required[str]
    """The name of the custom Scheduled Event Configuration.

    This must be in the format:

    - scheduled._name of entity_._custom event name_

    For example:

    - `scheduled.bill.endDateEvent`
    """

    offset: Required[int]
    """
    The offset in days from the specified DateTime field on the referenced entity
    when the scheduled Event will trigger.
    """

    version: int
    """The version number of the scheduled event configuration:

    - **Create entity**: Not valid for initial insertion - do not use for Create. On
      initial Create, version is set at 1 and listed in the response.
    - **Update Entity**: On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["DebitReasonUpdateParams"]


class DebitReasonUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    name: Required[str]
    """The name of the entity."""

    archived: bool
    """A Boolean TRUE / FALSE flag indicating whether the entity is archived.

    An entity can be archived if it is obsolete.

    - TRUE - the entity is in the archived state.
    - FALSE - the entity is not in the archived state.
    """

    code: str
    """The short code for the entity."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AccountGetChildrenParams"]


class AccountGetChildrenParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    next_token: Annotated[Optional[str], PropertyInfo(alias="nextToken")]

    page_size: Annotated[Optional[int], PropertyInfo(alias="pageSize")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ...types.users import invitation_list_params, invitation_create_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.users.invitation_response import InvitationResponse

__all__ = ["InvitationsResource", "AsyncInvitationsResource"]


class InvitationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InvitationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return InvitationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvitationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return InvitationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        email: str,
        first_name: str,
        last_name: str,
        contact_number: str | NotGiven = NOT_GIVEN,
        dt_end_access: Union[str, datetime] | NotGiven = NOT_GIVEN,
        dt_expiry: Union[str, datetime] | NotGiven = NOT_GIVEN,
        m3ter_user: bool | NotGiven = NOT_GIVEN,
        permission_policy_ids: List[str] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InvitationResponse:
        """
        Invite a new user to your Organization.

        This sends an email to someone inviting them to join your m3ter Organization.

        Args:
          email

          first_name

          last_name

          contact_number

          dt_end_access: The date when access will end for the user _(in ISO-8601 format)_. Leave blank
              for no end date, which gives the user permanent access.

          dt_expiry: The date when the invite expires _(in ISO-8601 format)_. After this date the
              invited user can no longer accept the invite. By default, any invite is valid
              for 30 days from the date the invite is sent.

          m3ter_user

          permission_policy_ids: The IDs of the permission policies the invited user has been assigned. This
              controls the access rights and privileges that this user will have when working
              in the m3ter Organization.

          version

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/invitations",
            body=maybe_transform(
                {
                    "email": email,
                    "first_name": first_name,
                    "last_name": last_name,
                    "contact_number": contact_number,
                    "dt_end_access": dt_end_access,
                    "dt_expiry": dt_expiry,
                    "m3ter_user": m3ter_user,
                    "permission_policy_ids": permission_policy_ids,
                    "version": version,
                },
                invitation_create_params.InvitationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvitationResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InvitationResponse:
        """
        Retrieve the specified invitation with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/invitations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvitationResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[InvitationResponse]:
        """
        Retrieve a list of all invitations in the Organization.

        Args:
          next_token: `nextToken` for multi page retrievals.

          page_size: Number of invitations to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/invitations",
            page=SyncCursor[InvitationResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    invitation_list_params.InvitationListParams,
                ),
            ),
            model=InvitationResponse,
        )


class AsyncInvitationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInvitationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvitationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvitationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncInvitationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        email: str,
        first_name: str,
        last_name: str,
        contact_number: str | NotGiven = NOT_GIVEN,
        dt_end_access: Union[str, datetime] | NotGiven = NOT_GIVEN,
        dt_expiry: Union[str, datetime] | NotGiven = NOT_GIVEN,
        m3ter_user: bool | NotGiven = NOT_GIVEN,
        permission_policy_ids: List[str] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InvitationResponse:
        """
        Invite a new user to your Organization.

        This sends an email to someone inviting them to join your m3ter Organization.

        Args:
          email

          first_name

          last_name

          contact_number

          dt_end_access: The date when access will end for the user _(in ISO-8601 format)_. Leave blank
              for no end date, which gives the user permanent access.

          dt_expiry: The date when the invite expires _(in ISO-8601 format)_. After this date the
              invited user can no longer accept the invite. By default, any invite is valid
              for 30 days from the date the invite is sent.

          m3ter_user

          permission_policy_ids: The IDs of the permission policies the invited user has been assigned. This
              controls the access rights and privileges that this user will have when working
              in the m3ter Organization.

          version

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/invitations",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "first_name": first_name,
                    "last_name": last_name,
                    "contact_number": contact_number,
                    "dt_end_access": dt_end_access,
                    "dt_expiry": dt_expiry,
                    "m3ter_user": m3ter_user,
                    "permission_policy_ids": permission_policy_ids,
                    "version": version,
                },
                invitation_create_params.InvitationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvitationResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InvitationResponse:
        """
        Retrieve the specified invitation with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/invitations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvitationResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[InvitationResponse, AsyncCursor[InvitationResponse]]:
        """
        Retrieve a list of all invitations in the Organization.

        Args:
          next_token: `nextToken` for multi page retrievals.

          page_size: Number of invitations to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/invitations",
            page=AsyncCursor[InvitationResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    invitation_list_params.InvitationListParams,
                ),
            ),
            model=InvitationResponse,
        )


class InvitationsResourceWithRawResponse:
    def __init__(self, invitations: InvitationsResource) -> None:
        self._invitations = invitations

        self.create = to_raw_response_wrapper(
            invitations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            invitations.retrieve,
        )
        self.list = to_raw_response_wrapper(
            invitations.list,
        )


class AsyncInvitationsResourceWithRawResponse:
    def __init__(self, invitations: AsyncInvitationsResource) -> None:
        self._invitations = invitations

        self.create = async_to_raw_response_wrapper(
            invitations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            invitations.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            invitations.list,
        )


class InvitationsResourceWithStreamingResponse:
    def __init__(self, invitations: InvitationsResource) -> None:
        self._invitations = invitations

        self.create = to_streamed_response_wrapper(
            invitations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            invitations.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            invitations.list,
        )


class AsyncInvitationsResourceWithStreamingResponse:
    def __init__(self, invitations: AsyncInvitationsResource) -> None:
        self._invitations = invitations

        self.create = async_to_streamed_response_wrapper(
            invitations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            invitations.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            invitations.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["M3terSignedCredentialsResponse"]


class M3terSignedCredentialsResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    destination: str
    """the system the integration is for"""

    type: str
    """the type of credentials"""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    api_key: Optional[str] = FieldInfo(alias="apiKey", default=None)
    """The API key provided by m3ter.

    This key is part of the credential set required for signing requests and
    authenticating with m3ter services.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this item."""

    destination_id: Optional[str] = FieldInfo(alias="destinationId", default=None)
    """the destinationId the integration is for"""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when this item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when this item was last modified _(in ISO-8601 format)_."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this item."""

    name: Optional[str] = None
    """the name of the credentials"""

    secret: Optional[str] = None
    """The secret associated with the API key.

    This secret is used in conjunction with the API key to generate a signature for
    secure authentication.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ResourceGroupListContentsResponse"]


class ResourceGroupListContentsResponse(BaseModel):
    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this item for the resource group."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the item was created for the resource group."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the resource group item was last modified."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this item for the resource group."""

    target_id: Optional[str] = FieldInfo(alias="targetId", default=None)
    """The UUID of the item."""

    target_type: Optional[Literal["ITEM", "GROUP"]] = FieldInfo(alias="targetType", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .bill_response import BillResponse

__all__ = ["BillSearchResponse"]


class BillSearchResponse(BaseModel):
    data: Optional[List[BillResponse]] = None
    """An array containing the list of requested Bills."""

    next_token: Optional[str] = FieldInfo(alias="nextToken", default=None)
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of Bills in a paginated list.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .job_list_params import JobListParams as JobListParams
from .file_upload_job_response import FileUploadJobResponse as FileUploadJobResponse
from .job_get_original_download_url_response import (
    JobGetOriginalDownloadURLResponse as JobGetOriginalDownloadURLResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.data_exports import destination_list_params, destination_create_params, destination_update_params
from ...types.data_exports.destination_create_response import DestinationCreateResponse
from ...types.data_exports.destination_delete_response import DestinationDeleteResponse
from ...types.data_exports.destination_update_response import DestinationUpdateResponse
from ...types.data_exports.destination_retrieve_response import DestinationRetrieveResponse
from ...types.data_exports.data_export_destination_response import DataExportDestinationResponse

__all__ = ["DestinationsResource", "AsyncDestinationsResource"]


class DestinationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DestinationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DestinationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DestinationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return DestinationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        bucket_name: str,
        code: str,
        iam_role_arn: str,
        name: str,
        partition_order: Optional[Literal["TYPE_FIRST", "TIME_FIRST"]] | NotGiven = NOT_GIVEN,
        prefix: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DestinationCreateResponse:
        """
        Create a new Export Destination to use for your Data Export Schedules or Ad-Hoc
        Data Exports.

        **NOTE:** Currently, you can only create Export Destinations using an S3 bucket
        on your AWS Account.

        Args:
          bucket_name: Name of the S3 bucket for the Export Destination.

          code: The code of the Export Destination.

          iam_role_arn: To enable m3ter to upload a Data Exports to your S3 bucket, the service has to
              assume an IAM role with PutObject permission for the specified `bucketName`.
              Create a suitable IAM role in your AWS account and enter ARN:

              **Formatting Constraints:**

              - The IAM role ARN must begin with "arn:aws:iam".
              - The general format required is:
                "arn:aws:iam::<aws account id>:role/<role name>". For example:
                "arn:aws:iam::922609978421:role/IAMRole636".
              - If the `iamRoleArn` used doesn't comply with this format, then an error
                message will be returned.

              **More Details:** For more details and examples of the Permission and Trust
              Policies you can use to create the required IAM Role ARN, see
              [Creating Data Export Destinations](https://www.m3ter.com/docs/guides/data-exports/creating-data-export-destinations)
              in our main User documentation.

          name: The name of the Export Destination.

          partition_order: Specify how you want the file path to be structured in your bucket destination -
              by Time first (Default) or Type first.

              Type is dependent on whether the Export is for Usage data or Operational data:

              - **Usage.** Type is `measurements`.
              - **Operational.** Type is one of the entities for which operational data
                exports are available, such as `account`, `commitment`, `meter`, and so on.

              Example for Usage Data Export using .CSV format:

              - Time first:
                `{bucketName}/{prefix}/orgId={orgId}/date=2025-01-27/hour=10/type=measurements/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
              - Type first:
                `{bucketName}/{prefix}/orgId={orgId}/type=measurements/date=2025-01-27/hour=10/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`

          prefix: Location in specified S3 bucket for the Export Destination. If you omit a
              `prefix`, then the root of the bucket will be used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/dataexports/destinations",
            body=maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "code": code,
                    "iam_role_arn": iam_role_arn,
                    "name": name,
                    "partition_order": partition_order,
                    "prefix": prefix,
                    "version": version,
                },
                destination_create_params.DestinationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DestinationCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DestinationRetrieveResponse:
        """
        Retrieve an Export Destination for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/dataexports/destinations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DestinationRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        bucket_name: str,
        code: str,
        iam_role_arn: str,
        name: str,
        partition_order: Optional[Literal["TYPE_FIRST", "TIME_FIRST"]] | NotGiven = NOT_GIVEN,
        prefix: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DestinationUpdateResponse:
        """
        Update an Export Destination for the given UUID.

        **NOTE:** Currently, only Export Destinations using an S3 bucket on your AWS
        Account are supported.

        Args:
          bucket_name: Name of the S3 bucket for the Export Destination.

          code: The code of the Export Destination.

          iam_role_arn: To enable m3ter to upload a Data Exports to your S3 bucket, the service has to
              assume an IAM role with PutObject permission for the specified `bucketName`.
              Create a suitable IAM role in your AWS account and enter ARN:

              **Formatting Constraints:**

              - The IAM role ARN must begin with "arn:aws:iam".
              - The general format required is:
                "arn:aws:iam::<aws account id>:role/<role name>". For example:
                "arn:aws:iam::922609978421:role/IAMRole636".
              - If the `iamRoleArn` used doesn't comply with this format, then an error
                message will be returned.

              **More Details:** For more details and examples of the Permission and Trust
              Policies you can use to create the required IAM Role ARN, see
              [Creating Data Export Destinations](https://www.m3ter.com/docs/guides/data-exports/creating-data-export-destinations)
              in our main User documentation.

          name: The name of the Export Destination.

          partition_order: Specify how you want the file path to be structured in your bucket destination -
              by Time first (Default) or Type first.

              Type is dependent on whether the Export is for Usage data or Operational data:

              - **Usage.** Type is `measurements`.
              - **Operational.** Type is one of the entities for which operational data
                exports are available, such as `account`, `commitment`, `meter`, and so on.

              Example for Usage Data Export using .CSV format:

              - Time first:
                `{bucketName}/{prefix}/orgId={orgId}/date=2025-01-27/hour=10/type=measurements/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
              - Type first:
                `{bucketName}/{prefix}/orgId={orgId}/type=measurements/date=2025-01-27/hour=10/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`

          prefix: Location in specified S3 bucket for the Export Destination. If you omit a
              `prefix`, then the root of the bucket will be used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/dataexports/destinations/{id}",
            body=maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "code": code,
                    "iam_role_arn": iam_role_arn,
                    "name": name,
                    "partition_order": partition_order,
                    "prefix": prefix,
                    "version": version,
                },
                destination_update_params.DestinationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DestinationUpdateResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[DataExportDestinationResponse]:
        """Retrieve a list of Export Destination entities.

        You can filter the list of
        Destinations returned by UUID.

        Args:
          ids: List of Export Destination UUIDs to retrieve.

          next_token: nextToken for multi page retrievals

          page_size: Number of returned Export Destinations to list per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/dataexports/destinations",
            page=SyncCursor[DataExportDestinationResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    destination_list_params.DestinationListParams,
                ),
            ),
            model=DataExportDestinationResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DestinationDeleteResponse:
        """
        Delete an Export Destination for the given UUID.

        **NOTE:** If you attempt to delete an Export Destination that is currently
        linked to a Data Export Schedule, an error message is returned and you won't be
        able to delete the Destination.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/dataexports/destinations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DestinationDeleteResponse,
        )


class AsyncDestinationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDestinationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDestinationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDestinationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncDestinationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        bucket_name: str,
        code: str,
        iam_role_arn: str,
        name: str,
        partition_order: Optional[Literal["TYPE_FIRST", "TIME_FIRST"]] | NotGiven = NOT_GIVEN,
        prefix: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DestinationCreateResponse:
        """
        Create a new Export Destination to use for your Data Export Schedules or Ad-Hoc
        Data Exports.

        **NOTE:** Currently, you can only create Export Destinations using an S3 bucket
        on your AWS Account.

        Args:
          bucket_name: Name of the S3 bucket for the Export Destination.

          code: The code of the Export Destination.

          iam_role_arn: To enable m3ter to upload a Data Exports to your S3 bucket, the service has to
              assume an IAM role with PutObject permission for the specified `bucketName`.
              Create a suitable IAM role in your AWS account and enter ARN:

              **Formatting Constraints:**

              - The IAM role ARN must begin with "arn:aws:iam".
              - The general format required is:
                "arn:aws:iam::<aws account id>:role/<role name>". For example:
                "arn:aws:iam::922609978421:role/IAMRole636".
              - If the `iamRoleArn` used doesn't comply with this format, then an error
                message will be returned.

              **More Details:** For more details and examples of the Permission and Trust
              Policies you can use to create the required IAM Role ARN, see
              [Creating Data Export Destinations](https://www.m3ter.com/docs/guides/data-exports/creating-data-export-destinations)
              in our main User documentation.

          name: The name of the Export Destination.

          partition_order: Specify how you want the file path to be structured in your bucket destination -
              by Time first (Default) or Type first.

              Type is dependent on whether the Export is for Usage data or Operational data:

              - **Usage.** Type is `measurements`.
              - **Operational.** Type is one of the entities for which operational data
                exports are available, such as `account`, `commitment`, `meter`, and so on.

              Example for Usage Data Export using .CSV format:

              - Time first:
                `{bucketName}/{prefix}/orgId={orgId}/date=2025-01-27/hour=10/type=measurements/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
              - Type first:
                `{bucketName}/{prefix}/orgId={orgId}/type=measurements/date=2025-01-27/hour=10/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`

          prefix: Location in specified S3 bucket for the Export Destination. If you omit a
              `prefix`, then the root of the bucket will be used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/dataexports/destinations",
            body=await async_maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "code": code,
                    "iam_role_arn": iam_role_arn,
                    "name": name,
                    "partition_order": partition_order,
                    "prefix": prefix,
                    "version": version,
                },
                destination_create_params.DestinationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DestinationCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DestinationRetrieveResponse:
        """
        Retrieve an Export Destination for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/dataexports/destinations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DestinationRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        bucket_name: str,
        code: str,
        iam_role_arn: str,
        name: str,
        partition_order: Optional[Literal["TYPE_FIRST", "TIME_FIRST"]] | NotGiven = NOT_GIVEN,
        prefix: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DestinationUpdateResponse:
        """
        Update an Export Destination for the given UUID.

        **NOTE:** Currently, only Export Destinations using an S3 bucket on your AWS
        Account are supported.

        Args:
          bucket_name: Name of the S3 bucket for the Export Destination.

          code: The code of the Export Destination.

          iam_role_arn: To enable m3ter to upload a Data Exports to your S3 bucket, the service has to
              assume an IAM role with PutObject permission for the specified `bucketName`.
              Create a suitable IAM role in your AWS account and enter ARN:

              **Formatting Constraints:**

              - The IAM role ARN must begin with "arn:aws:iam".
              - The general format required is:
                "arn:aws:iam::<aws account id>:role/<role name>". For example:
                "arn:aws:iam::922609978421:role/IAMRole636".
              - If the `iamRoleArn` used doesn't comply with this format, then an error
                message will be returned.

              **More Details:** For more details and examples of the Permission and Trust
              Policies you can use to create the required IAM Role ARN, see
              [Creating Data Export Destinations](https://www.m3ter.com/docs/guides/data-exports/creating-data-export-destinations)
              in our main User documentation.

          name: The name of the Export Destination.

          partition_order: Specify how you want the file path to be structured in your bucket destination -
              by Time first (Default) or Type first.

              Type is dependent on whether the Export is for Usage data or Operational data:

              - **Usage.** Type is `measurements`.
              - **Operational.** Type is one of the entities for which operational data
                exports are available, such as `account`, `commitment`, `meter`, and so on.

              Example for Usage Data Export using .CSV format:

              - Time first:
                `{bucketName}/{prefix}/orgId={orgId}/date=2025-01-27/hour=10/type=measurements/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
              - Type first:
                `{bucketName}/{prefix}/orgId={orgId}/type=measurements/date=2025-01-27/hour=10/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`

          prefix: Location in specified S3 bucket for the Export Destination. If you omit a
              `prefix`, then the root of the bucket will be used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/dataexports/destinations/{id}",
            body=await async_maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "code": code,
                    "iam_role_arn": iam_role_arn,
                    "name": name,
                    "partition_order": partition_order,
                    "prefix": prefix,
                    "version": version,
                },
                destination_update_params.DestinationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DestinationUpdateResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[DataExportDestinationResponse, AsyncCursor[DataExportDestinationResponse]]:
        """Retrieve a list of Export Destination entities.

        You can filter the list of
        Destinations returned by UUID.

        Args:
          ids: List of Export Destination UUIDs to retrieve.

          next_token: nextToken for multi page retrievals

          page_size: Number of returned Export Destinations to list per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/dataexports/destinations",
            page=AsyncCursor[DataExportDestinationResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    destination_list_params.DestinationListParams,
                ),
            ),
            model=DataExportDestinationResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DestinationDeleteResponse:
        """
        Delete an Export Destination for the given UUID.

        **NOTE:** If you attempt to delete an Export Destination that is currently
        linked to a Data Export Schedule, an error message is returned and you won't be
        able to delete the Destination.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/dataexports/destinations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DestinationDeleteResponse,
        )


class DestinationsResourceWithRawResponse:
    def __init__(self, destinations: DestinationsResource) -> None:
        self._destinations = destinations

        self.create = to_raw_response_wrapper(
            destinations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            destinations.retrieve,
        )
        self.update = to_raw_response_wrapper(
            destinations.update,
        )
        self.list = to_raw_response_wrapper(
            destinations.list,
        )
        self.delete = to_raw_response_wrapper(
            destinations.delete,
        )


class AsyncDestinationsResourceWithRawResponse:
    def __init__(self, destinations: AsyncDestinationsResource) -> None:
        self._destinations = destinations

        self.create = async_to_raw_response_wrapper(
            destinations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            destinations.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            destinations.update,
        )
        self.list = async_to_raw_response_wrapper(
            destinations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            destinations.delete,
        )


class DestinationsResourceWithStreamingResponse:
    def __init__(self, destinations: DestinationsResource) -> None:
        self._destinations = destinations

        self.create = to_streamed_response_wrapper(
            destinations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            destinations.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            destinations.update,
        )
        self.list = to_streamed_response_wrapper(
            destinations.list,
        )
        self.delete = to_streamed_response_wrapper(
            destinations.delete,
        )


class AsyncDestinationsResourceWithStreamingResponse:
    def __init__(self, destinations: AsyncDestinationsResource) -> None:
        self._destinations = destinations

        self.create = async_to_streamed_response_wrapper(
            destinations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            destinations.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            destinations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            destinations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            destinations.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.pricing_band import PricingBand

__all__ = ["PricingResponse"]


class PricingResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    accounting_product_id: Optional[str] = FieldInfo(alias="accountingProductId", default=None)

    aggregation_id: Optional[str] = FieldInfo(alias="aggregationId", default=None)
    """UUID of the Aggregation used to create the Pricing.

    Use this when creating a Pricing for a segmented aggregation.
    """

    aggregation_type: Optional[Literal["SIMPLE", "COMPOUND"]] = FieldInfo(alias="aggregationType", default=None)

    code: Optional[str] = None
    """Unique short code for the Pricing."""

    compound_aggregation_id: Optional[str] = FieldInfo(alias="compoundAggregationId", default=None)
    """UUID of the Compound Aggregation used to create the Pricing."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this item."""

    cumulative: Optional[bool] = None
    """
    Controls whether or not charge rates under a set of pricing bands configured for
    a Pricing are applied according to each separate band or at the highest band
    reached.

    The default value is **TRUE**.

    - When TRUE, at billing charge rates are applied according to each separate
      band.

    - When FALSE, at billing charge rates are applied according to highest band
      reached.
    """

    description: Optional[str] = None
    """Displayed on Bill line items."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when this item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when this item was last modified _(in ISO-8601 format)_."""

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """
    The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
    the Plan or Plan Template.

    If not specified, the Pricing remains active indefinitely.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this item."""

    minimum_spend: Optional[float] = FieldInfo(alias="minimumSpend", default=None)
    """
    The minimum spend amount per billing cycle for end customer Accounts on a Plan
    to which the Pricing is applied.
    """

    minimum_spend_bill_in_advance: Optional[bool] = FieldInfo(alias="minimumSpendBillInAdvance", default=None)
    """The default value is **FALSE**.

    - When TRUE, minimum spend is billed at the start of each billing period.

    - When FALSE, minimum spend is billed at the end of each billing period.

    _(Optional)_. Overrides the setting at Organization level for minimum spend
    billing in arrears/in advance.
    """

    minimum_spend_description: Optional[str] = FieldInfo(alias="minimumSpendDescription", default=None)
    """Minimum spend description _(displayed on the bill line item)_."""

    overage_pricing_bands: Optional[List[PricingBand]] = FieldInfo(alias="overagePricingBands", default=None)
    """
    The Prepayment/Balance overage pricing in pricing bands for the case of a
    **Tiered** pricing structure.
    """

    plan_id: Optional[str] = FieldInfo(alias="planId", default=None)
    """UUID of the Plan the Pricing is created for."""

    plan_template_id: Optional[str] = FieldInfo(alias="planTemplateId", default=None)
    """UUID of the Plan Template the Pricing was created for."""

    pricing_bands: Optional[List[PricingBand]] = FieldInfo(alias="pricingBands", default=None)

    segment: Optional[Dict[str, str]] = None
    """Name of the segment for which you are defining a Pricing.

    For each segment in a segmented aggregation, make a separate call using
    `aggregationId` parameter to update a Pricing.
    """

    segment_string: Optional[str] = FieldInfo(alias="segmentString", default=None)

    start_date: Optional[datetime] = FieldInfo(alias="startDate", default=None)
    """
    The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
    for the Plan of Plan Template.
    """

    tiers_span_plan: Optional[bool] = FieldInfo(alias="tiersSpanPlan", default=None)
    """The default value is **FALSE**.

    - If TRUE, usage accumulates over the entire period the priced Plan is active
      for the account, and is not reset for pricing band rates at the start of each
      billing period.

    - If FALSE, usage does not accumulate, and is reset for pricing bands at the
      start of each billing period.
    """

    type: Optional[Literal["DEBIT", "PRODUCT_CREDIT", "GLOBAL_CREDIT"]] = None
    """- **DEBIT**.

    Default setting. The amount calculated using the Pricing is added to the bill as
    a debit.

    - **PRODUCT_CREDIT**. The amount calculated using the Pricing is added to the
      bill as a credit _(negative amount)_. To prevent negative billing, the bill
      will be capped at the total of other line items for the _same_ Product.

    - **GLOBAL_CREDIT**. The amount calculated using the Pricing is added to the
      bill as a credit _(negative amount)_. To prevent negative billing, the bill
      will be capped at the total of other line items for the entire bill, which
      might include other Products the Account consumes.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PermissionPolicyAddToServiceUserResponse"]


class PermissionPolicyAddToServiceUserResponse(BaseModel):
    id: Optional[str] = None

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this principal permission."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime _(in ISO-8601 format)_ when the principal permission was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The DateTime _(in ISO-8601 format)_ when the principal permission was last
    modified.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this principal permission."""

    permission_policy_id: Optional[str] = FieldInfo(alias="permissionPolicyId", default=None)

    principal_id: Optional[str] = FieldInfo(alias="principalId", default=None)

    principal_type: Optional[Literal["USER", "USERGROUP", "SERVICEUSER", "SUPPORTUSERS"]] = FieldInfo(
        alias="principalType", default=None
    )

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .set_string import SetString as SetString
from .pricing_band import PricingBand as PricingBand
from .currency_conversion import CurrencyConversion as CurrencyConversion



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PlanGroupCreateParams"]


class PlanGroupCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    currency: Required[str]
    """Currency code for the PlanGroup (For example, USD)."""

    name: Required[str]
    """The name of the PlanGroup."""

    account_id: Annotated[str, PropertyInfo(alias="accountId")]
    """Optional.

    This PlanGroup is created as bespoke for the associated Account with this
    Account ID.
    """

    code: str
    """The short code representing the PlanGroup."""

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level, `customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    minimum_spend: Annotated[float, PropertyInfo(alias="minimumSpend")]
    """The minimum spend amount for the PlanGroup."""

    minimum_spend_accounting_product_id: Annotated[str, PropertyInfo(alias="minimumSpendAccountingProductId")]
    """Optional.

    Product ID to attribute the PlanGroup's minimum spend for accounting purposes.
    """

    minimum_spend_bill_in_advance: Annotated[bool, PropertyInfo(alias="minimumSpendBillInAdvance")]
    """A boolean flag that determines when the minimum spend is billed.

    This flag overrides the setting at Organizational level for minimum spend
    billing in arrears/in advance.

    - **TRUE** - minimum spend is billed at the start of each billing period.
    - **FALSE** - minimum spend is billed at the end of each billing period.
    """

    minimum_spend_description: Annotated[str, PropertyInfo(alias="minimumSpendDescription")]
    """Description of the minimum spend, displayed on the bill line item."""

    standing_charge: Annotated[float, PropertyInfo(alias="standingCharge")]
    """Standing charge amount for the PlanGroup."""

    standing_charge_accounting_product_id: Annotated[str, PropertyInfo(alias="standingChargeAccountingProductId")]
    """Optional.

    Product ID to attribute the PlanGroup's standing charge for accounting purposes.
    """

    standing_charge_bill_in_advance: Annotated[bool, PropertyInfo(alias="standingChargeBillInAdvance")]
    """A boolean flag that determines when the standing charge is billed.

    This flag overrides the setting at Organizational level for standing charge
    billing in arrears/in advance.

    - **TRUE** - standing charge is billed at the start of each billing period.
    - **FALSE** - standing charge is billed at the end of each billing period.
    """

    standing_charge_description: Annotated[str, PropertyInfo(alias="standingChargeDescription")]
    """Description of the standing charge, displayed on the bill line item."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import (
    external_mapping_list_params,
    external_mapping_create_params,
    external_mapping_update_params,
    external_mapping_list_by_m3ter_entity_params,
    external_mapping_list_by_external_entity_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.external_mapping_response import ExternalMappingResponse

__all__ = ["ExternalMappingsResource", "AsyncExternalMappingsResource"]


class ExternalMappingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExternalMappingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ExternalMappingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExternalMappingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return ExternalMappingsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        external_id: str,
        external_system: str,
        external_table: str,
        m3ter_entity: str,
        m3ter_id: str,
        integration_config_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExternalMappingResponse:
        """
        Creates a new External Mapping.

        This endpoint enables you to create a new External Mapping for the specified
        Organization. You need to supply a request body with the details of the new
        External Mapping.

        Args:
          external_id: The unique identifier (UUID) of the entity in the external system. This UUID
              should already exist in the external system.

          external_system: The name of the external system where the entity you are mapping resides.

          external_table: The name of the table in ther external system where the entity resides.

          m3ter_entity: The name of the m3ter entity that you are creating or modifying an External
              Mapping for. As an example, this could be an "Account".

          m3ter_id: The unique identifier (UUID) of the m3ter entity.

          integration_config_id: UUID of the integration config to link this mapping to

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/externalmappings",
            body=maybe_transform(
                {
                    "external_id": external_id,
                    "external_system": external_system,
                    "external_table": external_table,
                    "m3ter_entity": m3ter_entity,
                    "m3ter_id": m3ter_id,
                    "integration_config_id": integration_config_id,
                    "version": version,
                },
                external_mapping_create_params.ExternalMappingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalMappingResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExternalMappingResponse:
        """
        Retrieve an External Mapping with the given UUID.

        This endpoint enables you to retrieve the External Mapping with the specified
        UUID for a specific Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/externalmappings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalMappingResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        external_id: str,
        external_system: str,
        external_table: str,
        m3ter_entity: str,
        m3ter_id: str,
        integration_config_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExternalMappingResponse:
        """
        Updates an External Mapping with the given UUID.

        This endpoint enables you to update an existing External Mapping entity,
        identified by its UUID. You must supply a request body with the new details for
        the External Mapping.

        Args:
          external_id: The unique identifier (UUID) of the entity in the external system. This UUID
              should already exist in the external system.

          external_system: The name of the external system where the entity you are mapping resides.

          external_table: The name of the table in ther external system where the entity resides.

          m3ter_entity: The name of the m3ter entity that you are creating or modifying an External
              Mapping for. As an example, this could be an "Account".

          m3ter_id: The unique identifier (UUID) of the m3ter entity.

          integration_config_id: UUID of the integration config to link this mapping to

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/externalmappings/{id}",
            body=maybe_transform(
                {
                    "external_id": external_id,
                    "external_system": external_system,
                    "external_table": external_table,
                    "m3ter_entity": m3ter_entity,
                    "m3ter_id": m3ter_id,
                    "integration_config_id": integration_config_id,
                    "version": version,
                },
                external_mapping_update_params.ExternalMappingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalMappingResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        external_system_id: str | NotGiven = NOT_GIVEN,
        integration_config_id: str | NotGiven = NOT_GIVEN,
        m3ter_ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[ExternalMappingResponse]:
        """
        Retrieve a list of all External Mapping entities.

        This endpoint retrieves a list of all External Mapping entities for a specific
        Organization. The list can be paginated for better management, and supports
        filtering using the external system.

        Args:
          external_system_id: The name of the external system to use as a filter.

              For example, if you want to list only those external mappings created for your
              Organization for the Salesforce external system, use:

              `?externalSystemId=Salesforce`

          integration_config_id: ID of the integration config

          m3ter_ids: IDs for m3ter entities

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              External Mappings in a paginated list.

          page_size: Specifies the maximum number of External Mappings to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/externalmappings",
            page=SyncCursor[ExternalMappingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "external_system_id": external_system_id,
                        "integration_config_id": integration_config_id,
                        "m3ter_ids": m3ter_ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    external_mapping_list_params.ExternalMappingListParams,
                ),
            ),
            model=ExternalMappingResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExternalMappingResponse:
        """
        Delete an External Mapping with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/externalmappings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalMappingResponse,
        )

    def list_by_external_entity(
        self,
        external_id: str,
        *,
        org_id: str | None = None,
        system: str,
        external_table: str,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[ExternalMappingResponse]:
        """
        Retrieve a list of External Mapping entities for a specified external system
        entity.

        Use this endpoint to retrieve a list of External Mapping entities associated
        with a specific external system entity. The list can be paginated for easier
        management.

        Args:
          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              External Mappings in a paginated list.

          page_size: Specifies the maximum number of External Mappings to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not system:
            raise ValueError(f"Expected a non-empty value for `system` but received {system!r}")
        if not external_table:
            raise ValueError(f"Expected a non-empty value for `external_table` but received {external_table!r}")
        if not external_id:
            raise ValueError(f"Expected a non-empty value for `external_id` but received {external_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/externalmappings/externalid/{system}/{external_table}/{external_id}",
            page=SyncCursor[ExternalMappingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    external_mapping_list_by_external_entity_params.ExternalMappingListByExternalEntityParams,
                ),
            ),
            model=ExternalMappingResponse,
        )

    def list_by_m3ter_entity(
        self,
        m3ter_id: str,
        *,
        org_id: str | None = None,
        entity: str,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[ExternalMappingResponse]:
        """
        Retrieve a list of External Mapping entities for a specified m3ter entity.

        Use this endpoint to retrieve a list of External Mapping entities associated
        with a specific m3ter entity. The list can be paginated for easier management.

        Args:
          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              External Mappings in a paginated list.

          page_size: Specifies the maximum number of External Mappings to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not entity:
            raise ValueError(f"Expected a non-empty value for `entity` but received {entity!r}")
        if not m3ter_id:
            raise ValueError(f"Expected a non-empty value for `m3ter_id` but received {m3ter_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/externalmappings/external/{entity}/{m3ter_id}",
            page=SyncCursor[ExternalMappingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    external_mapping_list_by_m3ter_entity_params.ExternalMappingListByM3terEntityParams,
                ),
            ),
            model=ExternalMappingResponse,
        )


class AsyncExternalMappingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExternalMappingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExternalMappingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExternalMappingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncExternalMappingsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        external_id: str,
        external_system: str,
        external_table: str,
        m3ter_entity: str,
        m3ter_id: str,
        integration_config_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExternalMappingResponse:
        """
        Creates a new External Mapping.

        This endpoint enables you to create a new External Mapping for the specified
        Organization. You need to supply a request body with the details of the new
        External Mapping.

        Args:
          external_id: The unique identifier (UUID) of the entity in the external system. This UUID
              should already exist in the external system.

          external_system: The name of the external system where the entity you are mapping resides.

          external_table: The name of the table in ther external system where the entity resides.

          m3ter_entity: The name of the m3ter entity that you are creating or modifying an External
              Mapping for. As an example, this could be an "Account".

          m3ter_id: The unique identifier (UUID) of the m3ter entity.

          integration_config_id: UUID of the integration config to link this mapping to

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/externalmappings",
            body=await async_maybe_transform(
                {
                    "external_id": external_id,
                    "external_system": external_system,
                    "external_table": external_table,
                    "m3ter_entity": m3ter_entity,
                    "m3ter_id": m3ter_id,
                    "integration_config_id": integration_config_id,
                    "version": version,
                },
                external_mapping_create_params.ExternalMappingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalMappingResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExternalMappingResponse:
        """
        Retrieve an External Mapping with the given UUID.

        This endpoint enables you to retrieve the External Mapping with the specified
        UUID for a specific Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/externalmappings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalMappingResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        external_id: str,
        external_system: str,
        external_table: str,
        m3ter_entity: str,
        m3ter_id: str,
        integration_config_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExternalMappingResponse:
        """
        Updates an External Mapping with the given UUID.

        This endpoint enables you to update an existing External Mapping entity,
        identified by its UUID. You must supply a request body with the new details for
        the External Mapping.

        Args:
          external_id: The unique identifier (UUID) of the entity in the external system. This UUID
              should already exist in the external system.

          external_system: The name of the external system where the entity you are mapping resides.

          external_table: The name of the table in ther external system where the entity resides.

          m3ter_entity: The name of the m3ter entity that you are creating or modifying an External
              Mapping for. As an example, this could be an "Account".

          m3ter_id: The unique identifier (UUID) of the m3ter entity.

          integration_config_id: UUID of the integration config to link this mapping to

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/externalmappings/{id}",
            body=await async_maybe_transform(
                {
                    "external_id": external_id,
                    "external_system": external_system,
                    "external_table": external_table,
                    "m3ter_entity": m3ter_entity,
                    "m3ter_id": m3ter_id,
                    "integration_config_id": integration_config_id,
                    "version": version,
                },
                external_mapping_update_params.ExternalMappingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalMappingResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        external_system_id: str | NotGiven = NOT_GIVEN,
        integration_config_id: str | NotGiven = NOT_GIVEN,
        m3ter_ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ExternalMappingResponse, AsyncCursor[ExternalMappingResponse]]:
        """
        Retrieve a list of all External Mapping entities.

        This endpoint retrieves a list of all External Mapping entities for a specific
        Organization. The list can be paginated for better management, and supports
        filtering using the external system.

        Args:
          external_system_id: The name of the external system to use as a filter.

              For example, if you want to list only those external mappings created for your
              Organization for the Salesforce external system, use:

              `?externalSystemId=Salesforce`

          integration_config_id: ID of the integration config

          m3ter_ids: IDs for m3ter entities

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              External Mappings in a paginated list.

          page_size: Specifies the maximum number of External Mappings to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/externalmappings",
            page=AsyncCursor[ExternalMappingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "external_system_id": external_system_id,
                        "integration_config_id": integration_config_id,
                        "m3ter_ids": m3ter_ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    external_mapping_list_params.ExternalMappingListParams,
                ),
            ),
            model=ExternalMappingResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExternalMappingResponse:
        """
        Delete an External Mapping with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/externalmappings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalMappingResponse,
        )

    def list_by_external_entity(
        self,
        external_id: str,
        *,
        org_id: str | None = None,
        system: str,
        external_table: str,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ExternalMappingResponse, AsyncCursor[ExternalMappingResponse]]:
        """
        Retrieve a list of External Mapping entities for a specified external system
        entity.

        Use this endpoint to retrieve a list of External Mapping entities associated
        with a specific external system entity. The list can be paginated for easier
        management.

        Args:
          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              External Mappings in a paginated list.

          page_size: Specifies the maximum number of External Mappings to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not system:
            raise ValueError(f"Expected a non-empty value for `system` but received {system!r}")
        if not external_table:
            raise ValueError(f"Expected a non-empty value for `external_table` but received {external_table!r}")
        if not external_id:
            raise ValueError(f"Expected a non-empty value for `external_id` but received {external_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/externalmappings/externalid/{system}/{external_table}/{external_id}",
            page=AsyncCursor[ExternalMappingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    external_mapping_list_by_external_entity_params.ExternalMappingListByExternalEntityParams,
                ),
            ),
            model=ExternalMappingResponse,
        )

    def list_by_m3ter_entity(
        self,
        m3ter_id: str,
        *,
        org_id: str | None = None,
        entity: str,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ExternalMappingResponse, AsyncCursor[ExternalMappingResponse]]:
        """
        Retrieve a list of External Mapping entities for a specified m3ter entity.

        Use this endpoint to retrieve a list of External Mapping entities associated
        with a specific m3ter entity. The list can be paginated for easier management.

        Args:
          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              External Mappings in a paginated list.

          page_size: Specifies the maximum number of External Mappings to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not entity:
            raise ValueError(f"Expected a non-empty value for `entity` but received {entity!r}")
        if not m3ter_id:
            raise ValueError(f"Expected a non-empty value for `m3ter_id` but received {m3ter_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/externalmappings/external/{entity}/{m3ter_id}",
            page=AsyncCursor[ExternalMappingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    external_mapping_list_by_m3ter_entity_params.ExternalMappingListByM3terEntityParams,
                ),
            ),
            model=ExternalMappingResponse,
        )


class ExternalMappingsResourceWithRawResponse:
    def __init__(self, external_mappings: ExternalMappingsResource) -> None:
        self._external_mappings = external_mappings

        self.create = to_raw_response_wrapper(
            external_mappings.create,
        )
        self.retrieve = to_raw_response_wrapper(
            external_mappings.retrieve,
        )
        self.update = to_raw_response_wrapper(
            external_mappings.update,
        )
        self.list = to_raw_response_wrapper(
            external_mappings.list,
        )
        self.delete = to_raw_response_wrapper(
            external_mappings.delete,
        )
        self.list_by_external_entity = to_raw_response_wrapper(
            external_mappings.list_by_external_entity,
        )
        self.list_by_m3ter_entity = to_raw_response_wrapper(
            external_mappings.list_by_m3ter_entity,
        )


class AsyncExternalMappingsResourceWithRawResponse:
    def __init__(self, external_mappings: AsyncExternalMappingsResource) -> None:
        self._external_mappings = external_mappings

        self.create = async_to_raw_response_wrapper(
            external_mappings.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            external_mappings.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            external_mappings.update,
        )
        self.list = async_to_raw_response_wrapper(
            external_mappings.list,
        )
        self.delete = async_to_raw_response_wrapper(
            external_mappings.delete,
        )
        self.list_by_external_entity = async_to_raw_response_wrapper(
            external_mappings.list_by_external_entity,
        )
        self.list_by_m3ter_entity = async_to_raw_response_wrapper(
            external_mappings.list_by_m3ter_entity,
        )


class ExternalMappingsResourceWithStreamingResponse:
    def __init__(self, external_mappings: ExternalMappingsResource) -> None:
        self._external_mappings = external_mappings

        self.create = to_streamed_response_wrapper(
            external_mappings.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            external_mappings.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            external_mappings.update,
        )
        self.list = to_streamed_response_wrapper(
            external_mappings.list,
        )
        self.delete = to_streamed_response_wrapper(
            external_mappings.delete,
        )
        self.list_by_external_entity = to_streamed_response_wrapper(
            external_mappings.list_by_external_entity,
        )
        self.list_by_m3ter_entity = to_streamed_response_wrapper(
            external_mappings.list_by_m3ter_entity,
        )


class AsyncExternalMappingsResourceWithStreamingResponse:
    def __init__(self, external_mappings: AsyncExternalMappingsResource) -> None:
        self._external_mappings = external_mappings

        self.create = async_to_streamed_response_wrapper(
            external_mappings.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            external_mappings.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            external_mappings.update,
        )
        self.list = async_to_streamed_response_wrapper(
            external_mappings.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            external_mappings.delete,
        )
        self.list_by_external_entity = async_to_streamed_response_wrapper(
            external_mappings.list_by_external_entity,
        )
        self.list_by_m3ter_entity = async_to_streamed_response_wrapper(
            external_mappings.list_by_m3ter_entity,
        )
